<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\MainBundle\Entity;

use Chill\MainBundle\Doctrine\Model\TrackCreationInterface;
use Chill\MainBundle\Doctrine\Model\TrackCreationTrait;
use Chill\MainBundle\Doctrine\Model\TrackUpdateInterface;
use Chill\MainBundle\Doctrine\Model\TrackUpdateTrait;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\Common\Collections\ReadableCollection;
use Doctrine\ORM\Mapping as ORM;
use Ramsey\Uuid\Uuid;
use Ramsey\Uuid\UuidInterface;
use Symfony\Component\Validator\Constraints as Assert;

#[ORM\Entity]
#[ORM\Table(name: 'chill_main_saved_export')]
class SavedExport implements TrackCreationInterface, TrackUpdateInterface
{
    use TrackCreationTrait;

    use TrackUpdateTrait;

    #[Assert\NotBlank]
    #[ORM\Column(type: \Doctrine\DBAL\Types\Types::TEXT, nullable: false, options: ['default' => ''])]
    private string $description = '';

    #[ORM\Column(type: \Doctrine\DBAL\Types\Types::TEXT, nullable: false, options: ['default' => ''])]
    private string $exportAlias;

    #[ORM\Id]
    #[ORM\Column(name: 'id', type: 'uuid', unique: true)]
    #[ORM\GeneratedValue(strategy: 'NONE')]
    private UuidInterface $id;

    #[ORM\Column(type: \Doctrine\DBAL\Types\Types::JSON, nullable: false, options: ['default' => '[]'])]
    private array $options = [];

    #[Assert\NotBlank]
    #[ORM\Column(type: \Doctrine\DBAL\Types\Types::TEXT, nullable: false, options: ['default' => ''])]
    private string $title = '';

    #[ORM\ManyToOne(targetEntity: User::class)]
    private User $user;

    /**
     * @var Collection<int, User>
     */
    #[ORM\ManyToMany(targetEntity: User::class)]
    #[ORM\JoinTable(name: 'chill_main_saved_export_users')]
    private Collection $sharedWithUsers;

    /**
     * @var Collection<int, UserGroup>
     */
    #[ORM\ManyToMany(targetEntity: UserGroup::class)]
    #[ORM\JoinTable(name: 'chill_main_saved_export_usergroups')]
    private Collection $sharedWithGroups;

    public function __construct()
    {
        $this->id = Uuid::uuid4();
        $this->sharedWithUsers = new ArrayCollection();
        $this->sharedWithGroups = new ArrayCollection();
    }

    public function getDescription(): string
    {
        return $this->description;
    }

    public function getExportAlias(): string
    {
        return $this->exportAlias;
    }

    public function getId(): UuidInterface
    {
        return $this->id;
    }

    public function getOptions(): array
    {
        return $this->options;
    }

    public function getTitle(): string
    {
        return $this->title;
    }

    public function getUser(): User
    {
        return $this->user;
    }

    public function setDescription(?string $description): SavedExport
    {
        $this->description = (string) $description;

        return $this;
    }

    public function setExportAlias(string $exportAlias): SavedExport
    {
        $this->exportAlias = $exportAlias;

        return $this;
    }

    public function setOptions(array $options): SavedExport
    {
        $this->options = $options;

        return $this;
    }

    public function setTitle(?string $title): SavedExport
    {
        $this->title = (string) $title;

        return $this;
    }

    public function setUser(User $user): SavedExport
    {
        $this->user = $user;

        return $this;
    }

    public function addShare(User|UserGroup $shareUser): SavedExport
    {
        if ($shareUser instanceof User) {
            if (!$this->sharedWithUsers->contains($shareUser)) {
                $this->sharedWithUsers->add($shareUser);
            }
        } else {
            if (!$this->sharedWithGroups->contains($shareUser)) {
                $this->sharedWithGroups->add($shareUser);
            }
        }

        return $this;
    }

    public function removeShare(User|UserGroup $shareUser): SavedExport
    {
        if ($shareUser instanceof User) {
            $this->sharedWithUsers->removeElement($shareUser);
        } else {
            $this->sharedWithGroups->removeElement($shareUser);
        }

        return $this;
    }

    /**
     * @return ReadableCollection<int, User|UserGroup>
     */
    public function getShare(): ReadableCollection
    {
        return new ArrayCollection([
            ...$this->sharedWithUsers->toArray(),
            ...$this->sharedWithGroups->toArray(),
        ]);
    }

    /**
     * Determines if the user is shared with either directly or through a group.
     *
     * @param User $user the user to check
     *
     * @return bool returns true if the user is shared with directly or via group, otherwise false
     */
    public function isSharedWithUser(User $user): bool
    {
        if ($this->sharedWithUsers->contains($user)) {
            return true;
        }

        foreach ($this->sharedWithGroups as $group) {
            if ($group->contains($user)) {
                return true;
            }
        }

        return false;
    }
}
